#pragma once

#include "threads/CIOCriticalSection.h"
#include "db/Pools.h"

#include <windows.h>
#include <Sqltypes.h>

#include "utils/cstdint_support.h"

struct SqlConnection;

class DBConn
{
public:
    DBConn();               // L2CacheD 0x0040DAE0
    DBConn(Pools bbsPool);  // L2CacheD 0x0040DBE0
    ~DBConn();              // L2CacheD 0x0040DD50

    static void Init();  // L2CacheD 0x0040F4A0

    bool Fetch();                                                           // L2CacheD 0x0040EDF0
    void Bind(wchar_t* value, int length);                                  // L2CacheD 0x0040F0C0
    void Bind(char* value, int length);                                     // L2CacheD 0x0040F120
    void Bind(unsigned int* value);                                         // L2CacheD 0x0040F180
    void Bind(int* value);                                                  // L2CacheD 0x0040F1D0
    void Bind(double* value);                                               // L2CacheD 0x0040F220
    void Bind(uint8_t* pBinary, int size);                                  // L2CacheD 0x0040F270
    void BindNullable(uint8_t* pBinary, int size, SQLINTEGER* pIndicator);  // L2CacheD 0x0040F320
    void BindNullable(wchar_t* sValue, int size, SQLINTEGER* pIndicator);   // L2CacheD 0x0040F380
    void BindNullable(unsigned int* value, SQLINTEGER* pIndicator);         // L2CacheD 0x0040F3E0
    void BindNullable(int* value, SQLINTEGER* pIndicator);                  // L2CacheD 0x0040F440
    void ResetHtmt();                                                       // L2CacheD 0x0040F2D0
    bool Execute(const wchar_t* format, ...);                               // L2CacheD 0x0040F5C0
    int GetRowCount() const;                                                // L2CacheD 0x0040ED40

    SQLRETURN BindParam(SQLUSMALLINT index, SQLSMALLINT fParamType, SQLSMALLINT fCType, SQLSMALLINT fSqlType, SQLUINTEGER cbColDef, SQLSMALLINT ibScale, SQLPOINTER rgbValue, SQLINTEGER cbValueMax, SQLINTEGER* pcbValue);  // L2CacheD inlined
    SQLRETURN ExecuteOptimized(wchar_t* query);

private:
    static void AllocSQLPool();                                                                    // L2CacheD 0x0040DEC0
    static void AllocSQLPool2();                                                                   // L2CacheD 0x0040E110
    static void AllocSQLPool3();                                                                   // L2CacheD 0x0040E380
    static void Error(SQLSMALLINT handleType, SQLHANDLE handle, const wchar_t* query);             // L2CacheD 0x0040EF70
    static bool Login();                                                                           // L2CacheD 0x0040E940
    static bool LoadStrFromReg(wchar_t* str, const wchar_t* keyStr, const wchar_t* regEntry);      // L2CacheD 0x00465E70
    static void SaveStrToReg(const wchar_t* str, const wchar_t* keyStr, const wchar_t* regEntry);  // L2CacheD 0x00465FB0

    static BOOL CALLBACK loginDlgProc(HWND hDlg, UINT msgType, WPARAM wparam, LPARAM a4);     // L2CacheD 0x0040E600
    static BOOL CALLBACK loginBBSDlgProc(HWND hDlg, UINT msgType, WPARAM wparam, LPARAM a4);  // L2CacheD 0x0040E7A0

private:
    static int s_dbConnectionNum1;             // L2CacheD 0x006529D4
    static int s_dbConnectionNum2;             // L2CacheD 0x006529D0
    static int s_dbConnectionNum3;             // L2CacheD 0x006529D8
    static SqlConnection* s_sqlConnections1;   // L2CacheD 0x00652DEC
    static SqlConnection* s_sqlConnections2;   // L2CacheD 0x00652DE8
    static SqlConnection* s_sqlConnections3;   // L2CacheD 0x006529E0
    static wchar_t s_connStrBBS[256];          // L2CacheD 0x006529E8
    static wchar_t s_connStr[256];             // L2CacheD 0x00652BE8
    static SQLHANDLE s_sqlEnvHandle;           // L2CacheD 0x006529E4
    static SqlConnection* s_connectionsHead1;  // L2CacheD 0x00652DF0
    static SqlConnection* s_connectionsHead2;  // L2CacheD 0x00652DF4
    static SqlConnection* s_connectionsHead3;  // L2CacheD 0x00652DF8
    static CIOCriticalSection s_lock1;         // L2CacheD 0x00652E18
    static CIOCriticalSection s_lock2;         // L2CacheD 0x00652DFC

private:
    SQLHSTMT m_sqlHandler;
    int m_currentCollumn;
    SqlConnection* m_sqlConnection;
    Pools m_poolIndex;

    static const int QUERY_MAX_LEN = 512;
    wchar_t m_lastQuery[QUERY_MAX_LEN];
};
