#include "model/CCastle.h"

#include "ui/CLog.h"
#include "utils/dbg/StackGuard.h"
#include "utils/dbg/CallStack.h"
#include "memory/Allocator.h"

// L2CacheD 0x0049CFF0
long CCastle::s_nAlloc = ::InterlockedIncrement(&MemoryObject::s_refCountSlots);  // L2CacheD 0x00652980

// L2CacheD 0x0040D180
CCastle::CCastle(uint32_t castleId)
    : MemoryObject()
    , m_warehouse()
    , m_castleId(castleId)
{
    guard(L"CCastle::CCastle(int nCastleId)", L"CCastle::CCastle(int nCastleId) (exit)", TEXT(__FILE__), __LINE__);

    // CWareHouse* warehouse = new CWareHouse(); TODO: PORT ME
    // warehouse->m_ownerId = m_id;
    // warehouse->Init(WareHouseType_Castle, true);
    // m_warehouse.SetObject(warehouse);
    // warehouse->vRelease(__FILE__, __LINE__, ORT_CREATE_DELETE, 0);

    unguard();
}

// L2CacheD 0x0040BA20
CCastle::~CCastle()
{
    guard(L"CCastle::~CCastle()", L"CCastle::~CCastle() (exit)", TEXT(__FILE__), __LINE__);
    unguard();
}

// L2CacheD inlined
void* CCastle::operator new(size_t size)
{
    void* memory = g_allocator.allocate(size, 1, L"CCastle");

    ::InterlockedIncrement(&Allocator::g_allocatedObjects[CCastle::s_nAlloc]);

    Allocator::g_allocatedObjName[CCastle::s_nAlloc] = "CCastle";
    Allocator::g_allocatedMemory[CCastle::s_nAlloc] = sizeof(CCastle);

    return memory;
}

// L2CacheD inlined in 0x0040BFD0
void CCastle::operator delete(void* /*ptr*/)
{
    g_winlog.Add(LOG_ERROR, L"Never Call Oprator Delete!!! CCastle at file[%s], line[%d]", TEXT(__FILE__), __LINE__);
    CallStack::ReportCallStack();
}

// L2CacheD 0x0040B8F0
void CCastle::Release(const char* file, int line, ORTs /*nWorkType*/, bool /*flag*/)
{
    long refCountAfterRelease = ::InterlockedDecrement(&m_nRefCount);
    if (refCountAfterRelease > 0)
    {
        return;
    }

    if (refCountAfterRelease != 0)
    {
        if (refCountAfterRelease > -1)
        {
            return;
        }

        g_winlog.Add_Ansi(LOG_ERROR, "Invalid Memory (%d) on CCastle(%d)", refCountAfterRelease, m_identity);
        if (file)
        {
            g_winlog.Add_Ansi(LOG_ERROR, "           Caller : file[%s], line[%d]", file, line);
        }

        ReportOrt(0, 0);
        CallStack::ReportCallStack();

        return;
    }

    if (inTimer())
    {
        if (file)
        {
            g_winlog.Add_Ansi(LOG_ERROR, "deleting in-timer-object![CCastle] at file[%s], line[%d]", file, line);
            ReportOrt(0, 0);
            ::InterlockedDecrement(&Allocator::g_allocatedObjects[CCastle::s_nAlloc]);
        }
        else
        {
            g_winlog.Add_Ansi(LOG_ERROR, "deleting in-timer-object![CCastle]");
            ReportOrt(0, 0);
            ::InterlockedDecrement(&Allocator::g_allocatedObjects[CCastle::s_nAlloc]);
        }
        return;
    }

    this->~CCastle();

    g_allocator.deallocate(this);

    ::InterlockedDecrement(&Allocator::g_allocatedObjects[CCastle::s_nAlloc]);
}

// L2CacheD 0x0040C1C0
CWareHouseSP CCastle::GetWareHouse()
{
    guard(L"CWareHouseSP CCastle::GetWareHouse()", L"CWareHouseSP CCastle::GetWareHouse() (exit)", TEXT(__FILE__), __LINE__);

    throw "not Implemented!";  // TODO PORT ME
                               // CWareHouseSP::GetObjct(&m_warehouse, returnValue);
    unguard();

    // return returnValue;
}

uint32_t CCastle::id() const
{
    return m_castleId;
}
