#include "model/CPledgeCrestDb.h"

#include "db/DBConn.h"
#include "config/Config.h"
#include "model/CPledgeCrest.h"
#include "ui/CLog.h"
#include "utils/dbg/StackGuard.h"
#include "utils/dbg/CallStack.h"

#include <Sql.h>
#include <Sqlext.h>

// L2CacheD 0x0042AFD0
CPledgeCrestDb::CPledgeCrestDb()
{
    guard(L"CPledgeCrestDb::CPledgeCrestDb()", L"CPledgeCrestDb::CPledgeCrestDb() (exit)", TEXT(__FILE__), __LINE__);
    unguard();
}

// L2CacheD 0x0042B010
CPledgeCrestDb::~CPledgeCrestDb()
{
    guard(L"CPledgeCrestDb::~CPledgeCrestDb()", L"CPledgeCrestDb::~CPledgeCrestDb() (exit)", TEXT(__FILE__), __LINE__);
    unguard();
}

// L2CacheD 0x0042BCB0
uint32_t CPledgeCrestDb::CreatePledgeCrest(uint32_t bitmapSize, const uint8_t* bitmap)
{
    guard(L"unsigned int CPledgeCrestDb::CreatePledgeCrest(int nBitmapSize, const unsigned char *pBitmap)", L"unsigned int CPledgeCrestDb::CreatePledgeCrest(int nBitmapSize, const unsigned char *pBitmap) (exit)", TEXT(__FILE__), __LINE__);

    if (bitmapSize <= 0 || bitmapSize > CPledgeCrest::MAX_BITMAP_SIZE)
    {
        unguard();
        return 0;
    }

    DWORD before = ::GetTickCount();

    uint32_t crestId = 0;
    wchar_t sqlQuery[1024];
    //::swprintf(sqlQuery, query, bitmapSize); FIXED
    ::swprintf(sqlQuery, sizeof(sqlQuery) / sizeof(sqlQuery[0]), L"EXEC lin_SetPledgeCrest %d, ?", bitmapSize);

    SQLINTEGER cbValueMax = bitmapSize;

    DBConn sql;
    sql.Bind(&crestId);
    sql.BindParam(1u, SQL_CHAR, SQL_BINARY, SQL_BINARY, bitmapSize, 0, (SQLPOINTER)bitmap, cbValueMax, &cbValueMax);
    SQLRETURN sqlResult = sql.ExecuteOptimized(sqlQuery);

    DWORD duration = ::GetTickCount() - before;
    if (duration > Config::s_sqlExecLimit)
    {
        g_winlog.Add(LOG_ERROR, L"sql execution time [%d]ms, sql[%s]", duration, sqlQuery);
    }

    if (sqlResult != SQL_SUCCESS && sqlResult != SQL_SUCCESS_WITH_INFO)
    {
        unguard();
        return 0;
    }

    if (!sql.Fetch() || crestId == 0)
    {
        unguard();
        return 0;
    }

    CPledgeCrest* pledgeCrest = new CPledgeCrest(crestId, bitmapSize, bitmap);
    g_pledgeCrests.Insert(crestId, pledgeCrest, NULL, -1);

    unguard();
    return crestId;
}

// L2CacheD 0x0042BEF0
CPledgeCrestSP CPledgeCrestDb::LoadPledgeCrest(uint32_t crestId)
{
    guard(L"CPledgeCrestSP CPledgeCrestDb::LoadPledgeCrest(unsigned int nPledgeCrestId)", L"CPledgeCrestSP CPledgeCrestDb::LoadPledgeCrest(unsigned int nPledgeCrestId) (exit)", TEXT(__FILE__), __LINE__);

    uint32_t bitmapSize = 0;
    SQLINTEGER length = -1;
    uint8_t bitmap[CPledgeCrest::MAX_BITMAP_SIZE];

    DBConn sql;
    sql.Bind(&bitmapSize);
    sql.BindNullable(bitmap, CPledgeCrest::MAX_BITMAP_SIZE, &length);
    if (sql.Execute(L"EXEC lin_LoadPledgeCrest %d", crestId) && sql.Fetch())
    {
        CPledgeCrest* pledgeCrest = new CPledgeCrest(crestId, bitmapSize, bitmap);

        g_pledgeCrests.Insert(crestId, pledgeCrest);
        CPledgeCrestSP crestPtr = g_pledgeCrests.FindObject(crestId);

        unguard();
        return crestPtr;
    }

    unguard();

    return CPledgeCrestSP();
}
