#include "model/CPledgeWarDb.h"

#include "db/DBConn.h"
#include "model/CPledgeWar.h"
#include "model/CPledgeDb.h"
#include "ui/CLog.h"
#include "utils/dbg/StackGuard.h"
#include "utils/dbg/CallStack.h"
#include "utils/Utils.h"

CPledgeWarDb g_pledgeWarsDb;  // L2CacheD 0x019E63E8

// L2CacheD inlined
CPledgeWarDb::CPledgeWarDb()
    : m_wars()
{
}

// L2CacheD inlined
CPledgeWarDb::~CPledgeWarDb()
{
}

// L2CacheD 0x00431B40
CPledgeWarSP CPledgeWarDb::GetPledgeWar(uint32_t id)
{
    guard(L"CPledgeWarSP CPledgeWarDb::GetPledgeWar(uint32_t  nPledgeWarId)", L"CPledgeWarSP CPledgeWarDb::GetPledgeWar(uint32_t  nPledgeWarId) (exit)", TEXT(__FILE__), __LINE__);

    CPledgeWarSP war = m_wars.FindObject(id);
    if (war != NULL)
    {
        unguard();
        return war;
    }

    war = LoadPledgeWar(id);
    unguard();
    return war;
}

// L2CacheD 0x00431C70
uint32_t CPledgeWarDb::StartPledgeWar(uint32_t challengerId, uint32_t challengeeId, uint32_t beginTime)
{
    guard(L"uint32_t  CPledgeWarDb::StartPledgeWar(uint32_t  nChallengerPledgeId, uint32_t  nChallengeePledgeId, uint32_t  nWarBeginTime)", L"uint32_t  CPledgeWarDb::StartPledgeWar(uint32_t  nChallengerPledgeId, uint32_t  nChallengeePledgeId, uint32_t  nWarBeginTime) (exit)", TEXT(__FILE__), __LINE__);

    uint32_t warIdOut = 0;
    DBConn sql;
    sql.Bind(&warIdOut);
    if (sql.Execute(L"EXEC lin_StartPledgeWar %d, %d, %d", challengerId, challengeeId, beginTime))
    {
        sql.Fetch();
    }

    if (warIdOut == 0)
    {
        unguard();
        return 0;
    }

    wchar_t challengeePledgeName[52];
    ::memset(challengeePledgeName, 0, sizeof(challengeePledgeName));

    wchar_t challengerPledgeName[52];
    ::memset(challengerPledgeName, 0, sizeof(challengerPledgeName));

    CPledgeSP challengerPledge = g_pledgeDb.GetPledge(challengerId);
    if (challengerPledge != NULL)
    {
        challengerPledge->setNowWarId(warIdOut);
        ::wcscpy(challengerPledgeName, challengerPledge->name());
    }

    uint32_t challengeeCharId = 0;
    CPledgeSP challengeePledge = g_pledgeDb.GetPledge(challengeeId);
    if (challengeePledge != NULL)
    {
        challengeePledge->setNowWarId(warIdOut);
        ::wcscpy(challengeePledgeName, challengeePledge->name());
        challengeeCharId = challengeePledge->rullerId();
    }

    CPledgeWar* pledgeWar = new CPledgeWar(warIdOut, challengerId, challengeeId, beginTime, 0);
    m_wars.Insert(warIdOut, pledgeWar, NULL, -1);
    pledgeWar->Release(__FILE__, __LINE__, ORT_CREATE_DELETE, 0);

    //    CUserDb::GetUser_byId(&g_userDb, &challengeeUserPtr, challengeeCharId, true); TODO PORT ME
    //    v10 = challengeeUserPtr.m_object;
    //    LOBYTE(v47) = 5;
    //    if (challengeeUserPtr.m_object)
    //    {
    //        v11 = CSPointer::operator_indirect(&challengeeUserPtr);
    //        v12 = CUser::GetAccountName(v11);
    //        v13 = CSPointer::operator_indirect(&challengeeUserPtr);
    //        v14 = CUser::GetCharName(v13);
    //        v15 = warIdOut;
    //        v16 = CSPointer::operator_indirect(&challengeeUserPtr);
    //        v17 = CUser::GetLevel(v16);
    //        v18 = CSPointer::operator_indirect(&challengeeUserPtr);
    //        v19 = CUser::GetClass(v18);
    //        v20 = CSPointer::operator_indirect(&challengeeUserPtr);
    //        v21 = CUser::GetGender(v20);
    //        v22 = CSPointer::operator_indirect(&challengeeUserPtr);
    //        v23 = CUser::GetRace(v22);
    //        v24 = CSPointer::operator_indirect(&challengeeUserPtr);
    //        v25 = CUser::GetZ(v24);
    //        v26 = CSPointer::operator_indirect(&challengeeUserPtr);
    //        v27 = CUser::GetY(v26);
    //        v28 = CSPointer::operator_indirect(&challengeeUserPtr);
    //        v29 = CUser::GetX(v28);
    //        v30 = CSPointer::operator_indirect(&challengeeUserPtr);
    //        v31 = CUser::GetAccountID(v30);
    //        v32 = CSPointer::operator_indirect(&challengeeUserPtr);
    //        v33 = CUser::GetId(v32);
    //        g_winlog.Add(LOG_IN, L"%d,%u,%u,,,%d,%d,%d,%s,%s,,%d,%d,%d,%d,%d,%d,%d,%d,,,%s,%s,,,", LogId_BeginPledgeWar, v33, v31, v29, v27, v25, challengeePledgeName, challengerPledgeName, v23, v21, v19, v17, challengeeId, challengerId, beginTime, v15, v14, v12);
    //        v10 = challengeeUserPtr.m_object;
    //    }

    unguard();
    return warIdOut;
}

// L2CacheD 0x00432050
uint32_t CPledgeWarDb::StopPledgeWar(uint32_t proposerPledgeId, uint32_t proposeePledgeId, uint32_t warId, uint32_t warEndTime)
{
    guard(
        L"uint32_t  CPledgeWarDb::StoptPledgeWar(uint32_t  nProposerPledgeId, uint32_t  nProposeePledgeId, uint32_t  nWarId, uint32_t  nWarEndTime)",
        L"uint32_t  CPledgeWarDb::StoptPledgeWar(uint32_t  nProposerPledgeId, uint32_t  nProposeePledgeId, uint32_t  nWarId, uint32_t  nWarEndTime) (exit)",
        TEXT(__FILE__),
        __LINE__);

    uint32_t warIdOut = 0;
    DBConn sql;
    sql.Bind(&warIdOut);
    if (sql.Execute(L"EXEC lin_StopPledgeWar %d, %d, %d, %d", proposerPledgeId, proposeePledgeId, warId, warEndTime))
    {
        sql.Fetch();
    }

    if (warIdOut == 0)
    {
        unguard();
        return 0;
    }

    wchar_t proposerPledgeName[52];
    ::memset(proposerPledgeName, 0, sizeof(proposerPledgeName));

    wchar_t proposeePledgeName[52];
    ::memset(proposeePledgeName, 0, sizeof(proposeePledgeName));

    uint32_t proposerPledgeRullerId = 0;
    CPledgeSP proposerPledge = g_pledgeDb.GetPledge(proposerPledgeId);
    if (proposerPledge != NULL)
    {
        proposerPledge->setNowWarId(0);
        ::wcscpy(proposerPledgeName, proposerPledge->name());
        proposerPledgeRullerId = proposerPledge->rullerId();
    }

    CPledgeSP proposeePledge = g_pledgeDb.GetPledge(proposeePledgeId);
    if (proposeePledge != NULL)
    {
        proposeePledge->setNowWarId(0);
        ::wcscpy(proposeePledgeName, proposeePledge->name());
    }

    CPledgeWarSP war = GetPledgeWar(warId);
    if (war != NULL)
    {
        uint32_t duration = warEndTime - war->beginTime();
        //        CUserDb::GetUser_byId(&g_userDb, &proposerPledgeRullerPtr, proposerPledgeRullerId, 1);  TODO PORT ME
        //        proposerPledgeRuller = proposerPledgeRullerPtr.m_object;
        //        LOBYTE(v45) = 5;
        //        if (proposerPledgeRullerPtr.m_object)
        //        {
        //            v9 = CSPointer::operator_indirect(&proposerPledgeRullerPtr);
        //            v10 = CUser::GetAccountName(v9);
        //            v11 = CSPointer::operator_indirect(&proposerPledgeRullerPtr);
        //            v12 = CUser::GetCharName(v11);
        //            v13 = CSPointer::operator_indirect(&proposerPledgeRullerPtr);
        //            v14 = CUser::GetLevel(v13);
        //            v15 = CSPointer::operator_indirect(&proposerPledgeRullerPtr);
        //            v16 = CUser::GetClass(v15);
        //            v17 = CSPointer::operator_indirect(&proposerPledgeRullerPtr);
        //            v18 = CUser::GetGender(v17);
        //            v19 = CSPointer::operator_indirect(&proposerPledgeRullerPtr);
        //            v20 = CUser::GetRace(v19);
        //            v21 = CSPointer::operator_indirect(&proposerPledgeRullerPtr);
        //            v22 = CUser::GetZ(v21);
        //            v23 = CSPointer::operator_indirect(&proposerPledgeRullerPtr);
        //            v24 = CUser::GetY(v23);
        //            v25 = CSPointer::operator_indirect(&proposerPledgeRullerPtr);
        //            v26 = CUser::GetX(v25);
        //            v27 = CSPointer::operator_indirect(&proposerPledgeRullerPtr);
        //            v28 = CUser::GetAccountID(v27);
        //            v29 = CSPointer::operator_indirect(&proposerPledgeRullerPtr);
        //            v30 = CUser::GetId(v29);
        //            g_winlog.Add(LOG_IN, L"%d,%u,%u,,,%d,%d,%d,%s,%s,,%d,%d,%d,%d,%d,%d,%d,%d,,,%s,%s,,,", LogId_StopWar, v30, v28, v26, v24, v22, proposerPledgeName, proposeePledgeName, v20, v18, v16, v14, proposerPledgeId, proposeePledgeId, duration, warId, v12, v10);
        //            proposerPledgeRuller = proposerPledgeRullerPtr.m_object;
        //        }
    }

    m_wars.DeleteObject(warIdOut, NULL, -1);
    ReleaseRelatedSurrenderWar(warIdOut);

    unguard();
    return warIdOut;
}

// L2CacheD 0x00432440
uint32_t CPledgeWarDb::SurrenderPledgeWar(uint32_t proposerPledgeId, uint32_t proposeePledgeId, uint32_t warId, uint32_t warEndTime)
{
    guard(
        L"uint32_t  CPledgeWarDb::SurrenderPledgeWar(uint32_t  nProposerPledgeId, uint32_t  nProposeePledgeId, uint32_t  nWarId, uint32_t  nWarEndTime)",
        L"uint32_t  CPledgeWarDb::SurrenderPledgeWar(uint32_t  nProposerPledgeId, uint32_t  nProposeePledgeId, uint32_t  nWarId, uint32_t  nWarEndTime) (exit)",
        TEXT(__FILE__),
        __LINE__);

    uint32_t warIdOut = 0;
    DBConn sql;
    sql.Bind(&warIdOut);
    if (sql.Execute(L"EXEC lin_SurrenderPledgeWar %d, %d, %d, %d", proposerPledgeId, proposeePledgeId, warId, warEndTime))
    {
        sql.Fetch();
    }

    if (warIdOut == 0)
    {
        unguard();
        return 0;
    }

    wchar_t proposerPledgeName[52];
    ::memset(proposerPledgeName, 0, sizeof(proposerPledgeName));

    wchar_t proposeePledgeName[52];
    ::memset(proposeePledgeName, 0, sizeof(proposeePledgeName));

    CPledgeSP proposerPledge = g_pledgeDb.GetPledge(proposerPledgeId);
    if (proposerPledge != NULL)
    {
        proposerPledge->setNowWarId(0);
        ::wcscpy(proposerPledgeName, proposerPledge->name());
    }

    uint32_t proposeePledgeRullerId = 0;
    CPledgeSP proposeePledge = g_pledgeDb.GetPledge(proposeePledgeId);
    if (proposeePledge != NULL)
    {
        proposeePledge->setNowWarId(0);
        ::wcscpy(proposeePledgeName, proposeePledge->name());
        proposeePledgeRullerId = proposeePledge->rullerId();
    }

    CPledgeWarSP war = GetPledgeWar(warId);
    if (war != NULL)
    {
        //        uint32_t duration = warEndTime - war->beginTime();
        //        CUserDb::GetUser_byId(&g_userDb, &out, proposeePledgeRullerId, true); TODO: PORT ME
        //        v9 = out.m_object;
        //        LOBYTE(v45) = 5;
        //        if (out.m_object)
        //        {
        //            v10 = CSPointer::operator_indirect(&out);
        //            v11 = CUser::GetAccountName(v10);
        //            v12 = CSPointer::operator_indirect(&out);
        //            v13 = CUser::GetCharName(v12);
        //            v14 = CSPointer::operator_indirect(&out);
        //            v15 = CUser::GetLevel(v14);
        //            v16 = CSPointer::operator_indirect(&out);
        //            v17 = CUser::GetClass(v16);
        //            v18 = CSPointer::operator_indirect(&out);
        //            v19 = CUser::GetGender(v18);
        //            v20 = CSPointer::operator_indirect(&out);
        //            v21 = CUser::GetRace(v20);
        //            v22 = CSPointer::operator_indirect(&out);
        //            v23 = CUser::GetZ(v22);
        //            v24 = CSPointer::operator_indirect(&out);
        //            v25 = CUser::GetY(v24);
        //            v26 = CSPointer::operator_indirect(&out);
        //            v27 = CUser::GetX(v26);
        //            v28 = CSPointer::operator_indirect(&out);
        //            v29 = CUser::GetAccountID(v28);
        //            v30 = CSPointer::operator_indirect(&out);
        //            v31 = CUser::GetId(v30);
        //            g_winlog.Add(LOG_IN, L"%d,%u,%u,,,%d,%d,%d,%s,%s,,%d,%d,%d,%d,%d,%d,%d,%d,,,%s,%s,,,", LogId_SurrenderWar, v31, v29, v27, v25, v23, proposeePledgeName, proposerPledgeName, v21, v19, v17, v15, proposeePledgeId, proposerPledgeId, duration, warId, v13, v11);
        //            v9 = out.m_object;
        //        }
    }

    m_wars.DeleteObject(warIdOut, NULL, -1);
    ReleaseRelatedSurrenderWar(warIdOut);

    unguard();
    return warIdOut;
}

// L2CacheD 0x00432830
uint32_t CPledgeWarDb::FinishPledgeWar(uint32_t byTimeOut, uint32_t winnerPledgeId, uint32_t loserPledgeId, uint32_t warId, uint32_t warEndTime)
{
    guard(
        L"uint32_t  CPledgeWarDb::FinishPledgeWar(uint32_t  nByTimeOut, uint32_t  nWinnerPledgeId, uint32_t  nLoserPledgeId, uint32_t  nWarId, uint32_t  nWarEndTime)",
        L"uint32_t  CPledgeWarDb::FinishPledgeWar(uint32_t  nByTimeOut, uint32_t  nWinnerPledgeId, uint32_t  nLoserPledgeId, uint32_t  nWarId, uint32_t  nWarEndTime) (exit)",
        TEXT(__FILE__),
        __LINE__);

    uint32_t warIdOut = 0;
    DBConn sql;
    sql.Bind(&warIdOut);
    if (sql.Execute(L"EXEC lin_FinishPledgeWar %d, %d, %d, %d, %d", byTimeOut, winnerPledgeId, loserPledgeId, warId, warEndTime))
    {
        sql.Fetch();
    }

    if (warIdOut == 0)
    {
        unguard();
        return 0;
    }
    wchar_t winnerName[52];
    ::memset(winnerName, 0, sizeof(winnerName));

    wchar_t looserName[52];
    ::memset(looserName, 0, sizeof(looserName));

    uint32_t winnerRullerId = 0;
    CPledgeSP winnerPledge = g_pledgeDb.GetPledge(winnerPledgeId);
    if (winnerPledge != NULL)
    {
        winnerPledge->setNowWarId(0);
        ::wcscpy(winnerName, winnerPledge->name());
        winnerRullerId = winnerPledge->rullerId();
    }

    CPledgeSP looserPledge = g_pledgeDb.GetPledge(loserPledgeId);
    if (looserPledge != NULL)
    {
        looserPledge->setNowWarId(0);
        ::wcscpy(looserName, looserPledge->name());
    }

    CPledgeWarSP war = GetPledgeWar(warId);
    if (war != NULL)
    {
        uint32_t duration = warEndTime - war->beginTime();
        //            CUserDb::GetUser_byId(&g_userDb, &winnerPledgeRullerPtr, winnerRullerId, 1); TODO: PORT ME
        //            winnerPledgeRuller = winnerPledgeRullerPtr.m_object;
        //            LOBYTE(v68) = 5;
        //            if (winnerPledgeRullerPtr.m_object)
        //            {
        //                if (byTimeOut)
        //                {
        //                    winnerPledgeRuller1 = CSPointer::operator_indirect(&winnerPledgeRullerPtr);
        //                    winnerAccName = CUser::GetAccountName(winnerPledgeRuller1);
        //                    winnerPledgeRuller2 = CSPointer::operator_indirect(&winnerPledgeRullerPtr);
        //                    winnerCharName = CUser::GetCharName(winnerPledgeRuller2);
        //                    winnerPledgeRuller3 = CSPointer::operator_indirect(&winnerPledgeRullerPtr);
        //                    winnerLevel = CUser::GetLevel(winnerPledgeRuller3);
        //                    winnerPledgeRuller4 = CSPointer::operator_indirect(&winnerPledgeRullerPtr);
        //                    winnerClass = CUser::GetClass(winnerPledgeRuller4);
        //                    winnerPledgeRuller5 = CSPointer::operator_indirect(&winnerPledgeRullerPtr);
        //                    winnerGender = CUser::GetGender(winnerPledgeRuller5);
        //                    winnerPledgeRuller6 = CSPointer::operator_indirect(&winnerPledgeRullerPtr);
        //                    winnerRace = CUser::GetRace(winnerPledgeRuller6);
        //                    winnerPledgeRuller7 = CSPointer::operator_indirect(&winnerPledgeRullerPtr);
        //                    winnerLocZ = CUser::GetZ(winnerPledgeRuller7);
        //                    winnerPledgeRuller8 = CSPointer::operator_indirect(&winnerPledgeRullerPtr);
        //                    winnerLocY = CUser::GetY(winnerPledgeRuller8);
        //                    winnerPledgeRuller9 = CSPointer::operator_indirect(&winnerPledgeRullerPtr);
        //                    winnerLocX = CUser::GetX(winnerPledgeRuller9);
        //                    winnerPledgeRuller10 = CSPointer::operator_indirect(&winnerPledgeRullerPtr);
        //                    winnerAccId = CUser::GetAccountID(winnerPledgeRuller10);
        //                    winnerPledgeRuller11 = CSPointer::operator_indirect(&winnerPledgeRullerPtr);
        //                    winnerId = CUser::GetId(winnerPledgeRuller11);
        //                    g_winlog.Add(
        //                        LOG_IN,
        //                        L"%d,%u,%u,,,%d,%d,%d,%s,%s,,%d,%d,%d,%d,%d,%d,%d,%d,,,%s,%s,,,",
        //                        LogId_FinishWar,
        //                        winnerId,
        //                        winnerAccId,
        //                        winnerLocX,
        //                        winnerLocY,
        //                        winnerLocZ,
        //                        winnerName,
        //                        looserName,
        //                        winnerRace,
        //                        winnerGender,
        //                        winnerClass,
        //                        winnerLevel,
        //                        winnerPledgeId,
        //                        loserPledgeId,
        //                        duration,
        //                        warId,
        //                        winnerCharName,
        //                        winnerAccName);
        //                }
        //                else
        //                {
        //                    v10 = CSPointer::operator_indirect(&winnerPledgeRullerPtr);
        //                    v11 = CUser::GetAccountName(v10);
        //                    v12 = CSPointer::operator_indirect(&winnerPledgeRullerPtr);
        //                    v13 = CUser::GetCharName(v12);
        //                    v14 = CSPointer::operator_indirect(&winnerPledgeRullerPtr);
        //                    v15 = CUser::GetLevel(v14);
        //                    v16 = CSPointer::operator_indirect(&winnerPledgeRullerPtr);
        //                    v17 = CUser::GetClass(v16);
        //                    v18 = CSPointer::operator_indirect(&winnerPledgeRullerPtr);
        //                    v19 = CUser::GetGender(v18);
        //                    v20 = CSPointer::operator_indirect(&winnerPledgeRullerPtr);
        //                    v21 = CUser::GetRace(v20);
        //                    v22 = CSPointer::operator_indirect(&winnerPledgeRullerPtr);
        //                    v23 = CUser::GetZ(v22);
        //                    v24 = CSPointer::operator_indirect(&winnerPledgeRullerPtr);
        //                    v25 = CUser::GetY(v24);
        //                    v26 = CSPointer::operator_indirect(&winnerPledgeRullerPtr);
        //                    v27 = CUser::GetX(v26);
        //                    v28 = CSPointer::operator_indirect(&winnerPledgeRullerPtr);
        //                    v29 = CUser::GetAccountID(v28);
        //                    v30 = CSPointer::operator_indirect(&winnerPledgeRullerPtr);
        //                    v31 = CUser::GetId(v30);
        //                    g_winlog.Add(LOG_IN, L"%d,%u,%u,,,%d,%d,%d,%s,%s,,%d,%d,%d,%d,%d,%d,%d,%d,,,%s,%s,,,", LogId_WinWar, v31, v29, v27, v25, v23, winnerName, looserName, v21, v19, v17, v15, winnerPledgeId, loserPledgeId, duration, warId, v13, v11);
        //                }
        //                winnerPledgeRuller = winnerPledgeRullerPtr.m_object;
        //            }
    }

    m_wars.DeleteObject(warIdOut, NULL, -1);
    ReleaseRelatedSurrenderWar(warIdOut);

    unguard();
    return warIdOut;
}

// L2CacheD 0x00430E10
void CPledgeWarDb::ReleaseRelatedSurrenderWar(uint32_t pledgeWarId)
{
    guard(L"void CPledgeWarDb::ReleaseRelatedSurrenderWar(uint32_t  nPledgeWarId)", L"void CPledgeWarDb::ReleaseRelatedSurrenderWar(uint32_t  nPledgeWarId) (exit)", TEXT(__FILE__), __LINE__);

    uint32_t charId = 0;
    DBConn sql;
    sql.Bind(&charId);
    if (!sql.Execute(L"EXEC lin_DeleteSurrenderWar %d", pledgeWarId))
    {
        g_winlog.Add(LOG_ERROR, L"Error Occurred in Execute lin_DeleteSurrenderWar %d", pledgeWarId);
        unguard();
        return;
    }

    while (sql.Fetch())
    {
        //        CUserDb::GetUser_byId(&g_userDb, &userPtr, charId, true); PORT ME
        //        if (userPtr.m_object)
        //        {
        //            personalSurrenderWarId = CSPointer::operator_indirect(&userPtr)->m_d.m_personalSurrenderWarId;
        //            i = 10;
        //            do
        //            {
        //                if (*personalSurrenderWarId == pledgeWarId)
        //                {
        //                    *personalSurrenderWarId = 0;
        //                }
        //                ++personalSurrenderWarId;
        //                --i;
        //            } while (i);
        //        }
        //        else
        //        {
        //            g_winlog.Add(LOG_ERROR, L"User[%d] doesn't exist.", charId);
        //        }
    }

    unguard();
}

// L2CacheD 0x00431970
CPledgeWarSP CPledgeWarDb::LoadPledgeWar(uint32_t pledgeWarId)
{
    guard(L"CPledgeWarSP CPledgeWarDb::LoadPledgeWar(uint32_t  nPledgeWarId)", L"CPledgeWarSP CPledgeWarDb::LoadPledgeWar(uint32_t  nPledgeWarId) (exit)", TEXT(__FILE__), __LINE__);

    uint32_t challenger = 0;
    uint32_t challengee = 0;
    int beginTime = 0;
    int status = 0;

    DBConn sql;
    sql.Bind(&challenger);
    sql.Bind(&challengee);
    sql.Bind(&beginTime);
    sql.Bind(&status);
    if (sql.Execute(L"EXEC lin_LoadPledgeWar %d", pledgeWarId) && sql.Fetch())
    {
        CPledgeWar* pledgeWar = new CPledgeWar(pledgeWarId, challenger, challengee, beginTime, status);

        m_wars.Insert(pledgeWarId, pledgeWar, NULL, -1);
        pledgeWar->Release(__FILE__, __LINE__, ORT_CREATE_DELETE, 0);

        unguard();
        return m_wars.FindObject(pledgeWarId);
    }

    unguard();
    return CPledgeWarSP();
}
