#include "network/CLogCli.h"

#include "network/CLogSocket.h"
#include "network/packets/LogDPackets.h"
#include "utils/dbg/StackGuard.h"
#include "ui/CLog.h"
#include "config/Config.h"

CLogCli g_cliLog;  // L2CacheD 0x02684B0C

// L2CacheD 0x00484420
CLogCli::CLogCli()
    : m_logSocket(NULL)
    , m_lock()
{
    guard(L"CLogCli::CLogCli()", L"CLogCli::CLogCli() (exit)", TEXT(__FILE__), __LINE__);

    ::InitializeCriticalSection(&m_lock);

    unguard();
}

// L2CacheD 0x0049D9D0
CLogCli::~CLogCli()
{
    ::DeleteCriticalSection(&m_lock);
}

// L2CacheD 0x004835E0
void CLogCli::Job()
{
    while (true)
    {
        g_cliLog.Init();
        ::SuspendThread(g_winlog.cliLogThread());
    }
}

// L2CacheD 0x00484660
bool CLogCli::Init()
{
    guard(L"bool CLogCli::Init()", L"bool CLogCli::Init() (exit)", TEXT(__FILE__), __LINE__);  // FIXED: added

    ::EnterCriticalSection(&m_lock);

    if (g_winlog.IsSendable())
    {
        ::LeaveCriticalSection(&m_lock);
        unguard();
        return true;
    }
    g_winlog.Add(LOG_INF, L"CLogCli::Init()  Connect()");
    if (Connect())
    {
        g_winlog.Sendable(true);
        ::LeaveCriticalSection(&m_lock);
        unguard();
        return true;
    }

    g_winlog.Sendable(false);

    ::LeaveCriticalSection(&m_lock);

    unguard();
    return false;
}

// L2CacheD 0x00484470
bool CLogCli::Connect()
{
    guard(L"bool CLogCli::Connect()", L"bool CLogCli::Connect() (exit)", TEXT(__FILE__), __LINE__);

    SOCKET socket = ::socket(AF_INET, SOCK_STREAM, IPPROTO_IP);
    sockaddr_in name;
    name.sin_family = AF_INET;
    name.sin_addr = Config::s_logdIp;
    name.sin_port = ::htons(Config::s_logdPort);

    if (::connect(socket, (const struct sockaddr*)&name, sizeof(sockaddr_in)) == SOCKET_ERROR)
    {
        ::closesocket(socket);
        g_winlog.Add(LOG_ERROR, L"Can't connect to log server");
        unguard();
        return false;
    }

    g_winlog.Add(LOG_INF, L"Connected to log server");

    m_logSocket = new CLogSocket(socket, CLogSocket::s_packetHandlers);

    m_logSocket->Initialize();
    m_logSocket->Send("cd", LogD_SetCheckStatus, 1);

#ifdef L2CACHED
    m_logSocket->Send("cdd", LogD_ServerStarted, Config::s_worldId, ServerType_CacheD);
#elif defined L2LOGD
    m_logSocket->Send("cdd", LogD_ServerStarted, Config::s_worldId, ServerType_LogD);
#endif  // #ifdef L2CACHED

    unguard();

    return true;
}

// L2CacheD 0x004845E0
void CLogCli::LogSendMsg(LogType logType, const wchar_t* msg)
{
    guard(L"void CLogCli::LogSendMsg(LogType nLogType, wchar_t* szSendMsg)", L"void CLogCli::LogSendMsg(LogType nLogType, wchar_t* szSendMsg) (exit)", TEXT(__FILE__), __LINE__);

#ifdef L2CACHED
    m_logSocket->Send("cddS", LogD_Send, ServerType_CacheD, logType, msg);
#elif defined L2LOGD
    m_logSocket->Send("cddS", LogD_Send, ServerType_LogD, logType, msg);
#endif  // #ifdef L2CACHED

    unguard();
}
