#include "threads/CIOCriticalSection.h"

#include "threads/ThreadLocalStorage.h"
#include "utils/dbg/StackGuard.h"
#include "utils/dbg/LockStack.h"

// L2CacheD 0x00482A60
CIOCriticalSection::CIOCriticalSection()
    : m_criticalSection()
    , m_bActive(false)
{
    guard(L"CIOCriticalSection::CIOCriticalSection()", L"CIOCriticalSection::CIOCriticalSection() (exit)", TEXT(__FILE__), __LINE__);

    ::InitializeCriticalSection(&m_criticalSection);

    unguard();
}

// L2CacheD 0x00482AA0
CIOCriticalSection::CIOCriticalSection(uint32_t spinCount)
    : m_criticalSection()
    , m_bActive(false)
{
    guard(L"CIOCriticalSection::CIOCriticalSection(uint32_t spinCount)", L"CIOCriticalSection::CIOCriticalSection(uint32_t spinCount) (exit)", TEXT(__FILE__), __LINE__);

    ::InitializeCriticalSectionAndSpinCount(&m_criticalSection, spinCount);

    unguard();
}

// L2CacheD 0x00482AE0
CIOCriticalSection::~CIOCriticalSection()
{
    guard(L"CIOCriticalSection::~CIOCriticalSection()", L"CIOCriticalSection::~CIOCriticalSection() (exit)", TEXT(__FILE__), __LINE__);

    ::DeleteCriticalSection(&m_criticalSection);

    unguard();
}

// L2CacheD 0x00482B20
void CIOCriticalSection::Enter(const wchar_t* lockName, int line)
{
    if (line == 0)
    {
        line = reinterpret_cast<int>(this);
    }
    if (lockName == NULL)
    {
        lockName = L"CriticalSection Lock";
    }

    uint32_t threadIndex = tls::GetCurrentThreadIndex();
    LockStack& lockStack = g_stacks[threadIndex];
    lockStack.Push(0, 0, lockName, line);

    ::EnterCriticalSection(&m_criticalSection);
    m_bActive = true;
}

// L2CacheD 0x00482B80
void CIOCriticalSection::Leave(const wchar_t* unlockName, int line)
{
    m_bActive = false;
    if (line == 0)
    {
        line = reinterpret_cast<int>(this);
    }

    if (unlockName == NULL)
    {
        unlockName = L"CriticalSection Unlock";
    }

    uint32_t threadIndex = tls::GetCurrentThreadIndex();
    LockStack& lockStack = g_stacks[threadIndex];
    lockStack.Pop(0, 0, unlockName, line);

    ::LeaveCriticalSection(&m_criticalSection);
}
