#include <WinSock2.h>

#include "threads/Threads.h"

#include "model/CAccount.h"
#include "model/CAllianceDb.h"
#include "model/CItem.h"
#include "model/CPledgeDb.h"
#include "CIOObject.h"
#include "config/Config.h"
#include "db/DBConn.h"
#include "ui/CLog.h"
#include "utils/PerfMonitor.h"
#include "utils/dbg/StackGuard.h"
#include "utils/dbg/CExceptionInit.h"
#include "utils/dbg/Crash.h"
#include "threads/CIODeadlockChecker.h"
#include "threads/CThreadManager.h"
#include "threads/IOCP.h"
#include "threads/ThreadIndices.h"
#include "threads/ThreadLocalStorage.h"
#include "network/CLogCli.h"
#include "network/CIOBufferPool.h"
#include "network/CCacheDServer.h"
#include "network/CServerSocket.h"
#include "network/CNpcDbSocket.h"
#include "network/CAdminServer.h"

#include <ctime>
#include <cstdlib>

#include <process.h>

namespace Threads
{
// Performance monitor
HANDLE PerfMonitor_s_thread;  // L2CacheD 0x004F28B0

// IO thread
HANDLE s_timer;                     // L2CacheD 0x02684630
bool g_bTerminating;                // L2CacheD 0x02684628
std::vector<HANDLE> g_vHandle;      // L2CacheD 0x026846D4
std::vector<CIOObject*> g_vObject;  // L2CacheD 0x02684928

// L2CacheD 0x00409D40
void MessageLoop_common(HACCEL hAccTable)
{
    guard(L"void MessageLoop_common(HACCEL hAccTable)", L"void MessageLoop_common(HACCEL hAccTable) (exit)", TEXT(__FILE__), __LINE__);  // FIXED: added

    tagMSG msg;
    while (::GetMessageW(&msg, 0, 0, 0) != 0)
    {
        if (!::TranslateAcceleratorW(msg.hwnd, hAccTable, &msg))
        {
            ::TranslateMessage(&msg);
            ::DispatchMessageW(&msg);
        }
    }

    unguard();
}

// L2CacheD 0x00409DE0
void MessageLoop_mailing(HACCEL hAccTable)
{
    _BEFORE

    MessageLoop_common(hAccTable);

    _AFTER_FIN
}

// L2CacheD 0x0040A9B0
uint32_t __stdcall Threads::CacheDListeningThread(void*)
{
    CThreadManager::RegisterThisThread(ThreadIndex_ListenThread);
    CIOBufferPool* pool = new CIOBufferPool(100);
    g_IOBufferPools[tls::GetCurrentThreadIndex()] = pool;

    if (Config::ExceptionMailing())
    {
        Threads::CacheDListeningThread_mailing();
    }
    else
    {
        Threads::CacheDListeningThread_common();
    }

    return 0;
}

// L2CacheD 0x0040A7D0
void Threads::CacheDListeningThread_common()
{
    guard(L"void CacheDListeningThread_common()", L"CacheDListeningThread_common (exit)", TEXT(__FILE__), __LINE__);  // FIXED: added

    WSAData wsaData;
    int wsaStartupResult = ::WSAStartup(WINSOCK_VERSION, &wsaData);
    if (wsaStartupResult != 0)
    {
        g_winlog.Add_Ansi(LOG_ERROR, "WSAStartup error 0x%x", wsaStartupResult);
        unguard();
        return;
    }
    DBConn::Init();
    CItem::Init();
    g_allianceDb.Init();
    g_pledgeDb.Init();
    CAccount::Init();

    uint32_t ioThreadNumber = CThreadManager::GetMaxIOThreadNumber();
    Threads::CreateIOThread(ioThreadNumber);

    // g_adminServer.Run(2009);  // FIXED
    g_adminServer.Run(Config::s_adminServerPort);
    g_winlog.Add_Ansi(LOG_INF, "Server Runnig");

    // g_npcDbServer.Run(2008, &CNpcDbSocket::Allocate, false, 0, 0);  // FIXED
    g_npcDbServer.Run(Config::s_npcServerPort, &CNpcDbSocket::Allocate, false, 0, 0);

    // g_gameDbServer.Run(2006, &CServerSocket::Allocate, false, 0, 0);  // FIXED
    g_gameDbServer.Run(Config::s_gameServerPort, &CServerSocket::Allocate, false, 0, 0);

    g_winlog.ConnectToLogD();

    if (g_perfMonitor.Init())
    {
        unsigned int perfMonitorThread;
        Threads::PerfMonitor_s_thread = (HANDLE)_beginthreadex(NULL, 0, &Threads::PerfMonitorThread, 0, 0, &perfMonitorThread);
        CThreadManager::SetThread(ThreadIndex_PerfMonitor, (HANDLE)PerfMonitor_s_thread);
    }

    Threads::WaitThread();

    unguard();
}

// L2CacheD 0x0040A920
void Threads::CacheDListeningThread_mailing()
{
    _BEFORE

    Threads::CacheDListeningThread_common();

    _AFTER_FIN
}

// L2CacheD 0x0040A6B0
uint32_t __stdcall PerfMonitorThread(void*)
{
    CThreadManager::RegisterThisThread(ThreadIndex_PerfMonitor);

#ifdef L2CACHED
    g_IOBufferPools[tls::GetCurrentThreadIndex()] = new CIOBufferPool(100);
#endif  // #ifdef L2CACHED

    if (Config::ExceptionMailing())
    {
        PerfMonitorThread_mailing();
    }
    else
    {
        PerfMonitorThread_common();
    }

    return 0;
}

// L2CacheD 0x0040A610
void PerfMonitorThread_mailing()
{
    _BEFORE

    PerfMonitorThread_common();

    _AFTER_FIN
}

// // L2CacheD 0x0040A6A0
void PerfMonitorThread_common()
{
    guard(L"void PerfMonitorThread_common()", L"void PerfMonitorThread_common() (exit)", TEXT(__FILE__), __LINE__);  // FIXED: added

    PerfMonitor::Job();

    unguard();
}

// L2CacheD 0x00481DF0
void CreateIOThread(uint32_t maxIOThread)
{
    guard(L"void CreateIOThread(uint32_t maxIOThread)", L"void CreateIOThread(uint32_t maxIOThread) (exit)", TEXT(__FILE__), __LINE__);

    Threads::g_bTerminating = 0;
    Threads::s_timer = ::CreateEventW(0, 0, 0, 0);

    IOCP::g_hCompletionPort = ::CreateIoCompletionPort((HANDLE)INVALID_HANDLE_VALUE, 0, 0, 0);
    IOCP::g_hCompletionPortExtra = ::CreateIoCompletionPort((HANDLE)INVALID_HANDLE_VALUE, 0, 0, 0);
    IOCP::g_hBbsCompletionPort = ::CreateIoCompletionPort((HANDLE)INVALID_HANDLE_VALUE, 0, 0, 0);

    SYSTEM_INFO systemInfo;
    ::GetSystemInfo(&systemInfo);
    int numberOfProcessors = systemInfo.dwNumberOfProcessors;

    for (uint32_t i = 0; i < maxIOThread; ++i)
    {
        unsigned int threadId;
        HANDLE thread = (HANDLE)_beginthreadex(NULL, 0, &Threads::IOThread, (void*)i, 0, &threadId);

        ThreadIndices index = static_cast<ThreadIndices>(i);
        CThreadManager::SetThread(index, thread);

        int affinityMask = 1 << (i % numberOfProcessors);
        ::SetThreadAffinityMask(thread, affinityMask);
    }

    unguard();
}

// L2CacheD 0x004818E0
void WaitThread()
{
    guard(L"void WaitThread()", L"void WaitThread() (exit)", TEXT(__FILE__), __LINE__);

    g_deadlock_checker.Init();

    bool sleep = g_vHandle.size() == 0;

    while (true)
    {
        guard(L"WaitThread: wait loop", L"WaitThread: wait loop (exit)", TEXT(__FILE__), __LINE__);

#ifdef L2CACHED  // TODO: POR ME
//        stackFramesSize = 1000 * *(_DWORD*)(tls + 8) + CallStack::s_stackFramesSizes[threadIndex2]++ % 1000u;
//        CallStack::s_stackFrames[0].frames[stackFramesSize].guardedFuncName = (const wchar_t*)-1;
#endif
        long dwTick = ::GetTickCount();

#ifdef L2CACHED
        if (dwTick > (g_deadlock_checker.lastTick() + 60 * 1000))
#else
        if (dwTick > (g_deadlock_checker.lastTick() + 180 * 1000))
#endif
        {
            guard(L"!!! deadlock or super-lag detected !!!", L"(exit)", TEXT(__FILE__), __LINE__);
            g_winlog.Add(LOG_ERROR, L"!!! deadlock or super-lag detected!!!  (diff %d sec) ", (dwTick - g_deadlock_checker.lastTick()) / 1000);

#ifdef NDEBUG
            CRASH;
#endif
            unguard();
        }

        LONG nMinNextTime = ::GetTickCount() + 2 * CIO_DEADLOCK_CHECKER_INTERVAL;
        LONG nWait = 2 * CIO_DEADLOCK_CHECKER_INTERVAL;

        CIOObject::TimerDispatch();

        if ((nMinNextTime - ::GetTickCount()) >= 100)
        {
            nWait = 100;
        }
        else
        {
            nWait = nMinNextTime - ::GetTickCount();
        }

        if (sleep)
        {
            ::Sleep(nWait);
        }
        else
        {
            DWORD dwWaitResult = ::WaitForMultipleObjects(g_vHandle.size(), &g_vHandle[0], FALSE, nWait);
            if ((dwWaitResult >= WAIT_OBJECT_0) && (dwWaitResult < WAIT_OBJECT_0 + g_vHandle.size()))
            {
                dwWaitResult -= WAIT_OBJECT_0;
                CIOObject* object = g_vObject[dwWaitResult];
                object->OnWaitCallback();
            }
        }

        if (Threads::g_bTerminating)
        {
            CRASH;
        }

        unguard();
    }
}

// L2CacheD 0x00481D20
void IOThread_mailing(ThreadIndices index)
{
    _BEFORE

    IOThread_common(index);

    _AFTER_FIN
}

// L2CacheD 0x00481AE0
void IOThread_common(ThreadIndices threadIndex)
{
    guard(L"void Threads::IOThread_common(ThreadIndices threadIndex)", L"void Threads::IOThread_common(ThreadIndices threadIndex) (exit)", TEXT(__FILE__), __LINE__);  // FIXED: added

#ifdef L2CACHED
    int ioBufferCount = ::GetPrivateProfileIntW(L"Setting", L"IOBufferCount", 40000, L".\\l2cache.ini");
#else
    int ioBufferCount = ::GetPrivateProfileIntW(L"Setting", L"IOBufferCount", 20000, L".\\l2server.ini");
#endif  // #ifdef L2CACHED

    CIOBufferPool* pool = new CIOBufferPool(ioBufferCount / CThreadManager::GetMaxIOThreadNumber());
    g_IOBufferPools[tls::GetCurrentThreadIndex()] = pool;

    DWORD nextSyncTime = ::GetTickCount();

#ifdef L2CACHED
    HANDLE completionPort = (threadIndex <= ThreadIndex_IO8) ? IOCP::g_hCompletionPort : IOCP::g_hBbsCompletionPort;
#else
    HANDLE completionPort = IOCP::g_hCompletionPort;
#endif  // #ifdef L2CACHED

    std::srand((int)std::time(NULL));

    static const int IOC_WAIT = 50;
    while (true)
    {
#ifdef L2CACHED
// TODO: PORT ME
//        int threadIndex = tls::GetCurrentThreadIndex();
//        stackFramesSize = 1000 * int + CallStack::s_stackFramesSizes[threadIndex_1]++ % 1000u;
//        CallStack::s_stackFrames[0].frames[stackFramesSize].guardedFuncName = (const wchar_t*)-1;
#endif  // #ifdef L2CACHED

        guard(L"Threads::IOThread_common while loop", L"Threads::IOThread_common while loop (exit)", TEXT(__FILE__), __LINE__);

        CIOObject* object = NULL;
        DWORD now = ::GetTickCount();
        DWORD waitTick;
        if (nextSyncTime >= now)
        {
            waitTick = nextSyncTime - now;
        }
        else
        {
            CIOObject::TimerDispatch();
            nextSyncTime = ::GetTickCount() + IOC_WAIT;
            waitTick = IOC_WAIT;
        }

        // ::InterlockedIncrement(&CReporter::g_nRunningThread); UNUSED in CacheD

        DWORD dwTransferred;
        LPOVERLAPPED overlapped;
        BOOL success = ::GetQueuedCompletionStatus(completionPort, &dwTransferred, (PULONG_PTR)&object, &overlapped, waitTick);

        //::InterlockedDecrement(&CReporter::g_nRunningThread); UNUSED in CacheD

        // if (success || overlapped || object)
        if (object != NULL)
        {
            object->OnIOCallback(success, dwTransferred, overlapped);
        }

        unguard();
    }

    unguard();
}

// L2CacheD 0x00481D30
uint32_t __stdcall IOThread(void* threadIndex)
{
    int param = reinterpret_cast<int>(threadIndex);
    ThreadIndices index = static_cast<ThreadIndices>(param);
    CThreadManager::RegisterThisThread(index);

    if (Config::ExceptionMailing())
    {
        IOThread_mailing(index);
    }
    else
    {
        IOThread_common(index);
    }

    return 0;
}

// L2CacheD 0x00484130
uint32_t __stdcall CliLogThread(void*)
{
    CThreadManager::RegisterThisThread(ThreadIndex_CliLog);

    CIOBufferPool* pool = new CIOBufferPool(50);

    g_IOBufferPools[ThreadIndex_CliLog] = pool;

    if (Config::ExceptionMailing())
    {
        Threads::CliLogThread_mailing();
    }
    else
    {
        Threads::CliLogThread_common();
    }

    return 0;
}

// L2CacheD 0x004836A0
void CliLogThread_common()
{
    guard(L"void CliLogThread_common()", L"void CliLogThread_common() (exit)", TEXT(__FILE__), __LINE__);  // FIXED: added

    CLogCli::Job();  //  INLINED COMMON

    unguard();
}

// L2CacheD 0x00483610
void CliLogThread_mailing()
{
    _BEFORE

    Threads::CliLogThread_common();

    _AFTER_FIN
}

}  // namespace Threads
