#pragma once

#include "logger/LogType.h"
#include "logger/ServerTypes.h"
#include "logger/FileBuffers.h"
#include "threads/ThreadIndices.h"

#include <windows.h>
#include <varargs.h>

class CLog
{
public:
    CLog(int size);  // L2CacheD 0x00484190
    ~CLog();         // L2CacheD 0x00482F00

    void ConnectToLogD();  // L2CacheD 0x00483300

    void Set(HWND logWnd);                                     // L2CacheD 0x00482FD0
    void Redraw();                                             // L2CacheD 0x004830E0
    void Enable(bool enable);                                  // L2CacheD 0x00483270
    void Sendable(bool sendable);                              // L2CacheD 0x004832C0
    bool IsSendable() const;                                   // inlined
    void FlushLogFile();                                       // L2CacheD 0x00483340
    void Add_Ansi(LogType severity, const char* format, ...);  // L2CacheD 0x004842D0 TODO: FIX ME!
    void Add(LogType severity, const wchar_t* format, ...);    // L2CacheD 0x00484340
    void ForceNewFile();                                       // inlined in binary
    HANDLE cliLogThread() const;

private:
    bool CanAppend(int year, int month, int day, int hour, bool half);         // L2CacheD 0x00482F80
    void CreateNewLogFile(int year, int month, int day, int hour, bool half);  // L2CacheD 0x004833C0
    void LockBuffer(int bufferIndex);                                          // L2CacheD 0x00483010
    void UnlockBuffer(int bufferIndex);                                        // L2CacheD 0x00483030
    void AddV(LogType severity, const char* format, va_list params);           // L2CacheD 0x004836B0
    void AddV(LogType severity, const wchar_t* format, va_list params);        // L2CacheD 0x00483C30

private:
    struct LogEntry;
    struct FileName
    {
        wchar_t path[MAX_PATH];
    };

public:
    static const wchar_t* s_serverTypes[7];             // 0x004F15FC
    static const wchar_t* s_logCategories[8];           // 0x004F15DC
    static int s_stackSizes[ThreadIndex_MaxCount + 1];  // 0x02684960

    HWND m_window;
    bool m_enabled;
    bool m_sendable;
    // bool m_enabled3; unused
    const int m_bufferLength;
    int m_currentLine;
    LogEntry* m_outputBuffer;
    CRITICAL_SECTION m_criticalSection;
    RECT m_loginWndRect;
    SIZE m_textSize;
    HGDIOBJ m_font;
    HBRUSH m_brush;
    unsigned int m_threadAddr;
    HANDLE m_logThread;
    int m_year;
    int m_month;
    int m_day;
    int m_hour;
    bool m_half;
    int m_worldID;
    ServerTypes m_logComponent;
    HANDLE m_fileBuffers[FileBuffers_BuffersCount];
    CRITICAL_SECTION m_bufferLocks[FileBuffers_BuffersCount];
#ifdef L2LOG
    FileName m_fileNames[FileBuffers_BuffersCount];
#endif
    char m_createNewFile;
};

extern CLog g_winlog;  // L2CacheD 0x026849B8
