#include "network/CIOBuffer.h"

#include "utils/dbg/Crash.h"
#include "utils/dbg/StackGuard.h"
#include "network/CIOBufferPool.h"
#include "threads/ThreadLocalStorage.h"

// 0x00408950
CIOBuffer::CIOBuffer()
    : m_dwSize(0)
    , m_pNext(NULL)
    , m_nRefCount(0)
    , m_threadIndex(0)
{
    guard(L"CIOBuffer::CIOBuffer()");
    unguard();
}

CIOBuffer::~CIOBuffer()
{
}

// 0x00409460
CIOBuffer* CIOBuffer::Alloc()
{
    uint32_t threadIndex = tls::GetCurrentThreadIndex();
    CIOBufferPool* pool = g_IOBufferPools[threadIndex];

    ::InterlockedDecrement(&pool->m_freeSlots);

    //++pool->m_allocIndex;  FIXED
    int allocIndex = ::InterlockedIncrement(&pool->m_allocIndex);

    LONG id = allocIndex % pool->getPoolMaxSize();
    CIOBuffer* buffer = pool->m_bufferIndexes[id];
    buffer->m_threadIndex = threadIndex;

    if (pool->m_freeSlots < 8)
    {
        guard(L"io_buffer_crash");
        CRASH;
        unguard();
    }

    buffer->m_dwSize = 0;
    buffer->m_pNext = 0;
    buffer->m_nRefCount = 1;

    return buffer;
}

// 0x00409500
void CIOBuffer::Free()
{
    CIOBufferPool* pool = g_IOBufferPools[m_threadIndex];
    ::InterlockedIncrement(&pool->m_freeSlots);

    LONG freeIndex = ::InterlockedIncrement(&pool->m_freeIndex);
    LONG id = freeIndex % pool->getPoolMaxSize();

    pool->m_bufferIndexes[id] = this;
}

long CIOBuffer::RefCount() const
{
    return m_nRefCount;
}

// inlined
void CIOBuffer::AddRef()
{
    ::InterlockedIncrement(&m_nRefCount);
}

// 0x00409CB0
void CIOBuffer::Release()
{
    ::InterlockedDecrement(&m_nRefCount);
    if (m_nRefCount == 0)
    {
        CIOBuffer::Free();
    }
}
