#include "threads/CIOCriticalSection.h"

#include "threads/ThreadLocalStorage.h"
#include "utils/dbg/StackGuard.h"
#include "utils/dbg/LockStack.h"

// 0x0040BC00
CIOCriticalSection::CIOCriticalSection()
    : m_criticalSection()
    , m_bActive(false)
{
    guard(L"CIOCriticalSection::CIOCriticalSection()");

    ::InitializeCriticalSection(&m_criticalSection);

    unguard();
}

// 0x0040BC30
CIOCriticalSection::CIOCriticalSection(uint32_t spinCount)
    : m_criticalSection()
    , m_bActive(false)
{
    guard(L"CIOCriticalSection::CIOCriticalSection(DWORD dwSpinCount)");

    ::InitializeCriticalSectionAndSpinCount(&m_criticalSection, spinCount);

    unguard();
}

// 0x0040BC70
CIOCriticalSection::~CIOCriticalSection()
{
    guard(L"CIOCriticalSection::~CIOCriticalSection()");

    ::DeleteCriticalSection(&m_criticalSection);

    unguard();
}

// 0x0040BCA0
void CIOCriticalSection::Enter(const wchar_t* lockName, int line)
{
    if (line == 0)
    {
        line = reinterpret_cast<int>(this);
    }
    if (lockName == NULL)
    {
        lockName = L"CriticalSection Lock";
    }

    uint32_t threadIndex = tls::GetCurrentThreadIndex();
    LockStack& lockStack = g_stacks[threadIndex];
    lockStack.Push(0, 0, lockName, line);

    ::EnterCriticalSection(&m_criticalSection);
    m_bActive = true;
}

// 0x0040BD00
void CIOCriticalSection::Leave(const wchar_t* unlockName, int line)
{
    m_bActive = false;
    if (line == 0)
    {
        line = reinterpret_cast<int>(this);
    }

    if (unlockName == NULL)
    {
        unlockName = L"CriticalSection Unlock";
    }

    uint32_t threadIndex = tls::GetCurrentThreadIndex();
    LockStack& lockStack = g_stacks[threadIndex];
    lockStack.Pop(0, 0, unlockName, line);

    ::LeaveCriticalSection(&m_criticalSection);
}
