#include <winsock2.h>

#include "threads/Threads.h"

#include "threads/CIODeadlockChecker.h"
#include "threads/CThreadManager.h"
#include "threads/IOCP.h"
#include "threads/ThreadIndices.h"
#include "threads/ThreadLocalStorage.h"
#include "db/CSQLQueue.h"
#include "config/Config.h"
#include "utils/PerfMonitor.h"
#include "utils/dbg/Crash.h"
#include "utils/CBCPLogFile.h"
#include "utils/dbg/CExceptionInit.h"
#include "utils/dbg/StackGuard.h"
#include "network/CLogCli.h"
#include "network/CIOBufferPool.h"
#include "network/CServer.h"
#include "network/CServerSocket.h"

#include <ctime>
#include <process.h>

namespace Threads
{

// Performance monitor
HANDLE PerfMonitor_s_thread;  // 0x00455C1C

// BCP
HANDLE BCPThread_s_threads[10];  // 0x00455C70

// IO thread
HANDLE s_timer;                     // 0x01BD3BB8
bool g_bTerminating;                // 0x01BD3BBC
std::vector<HANDLE> g_vHandle;      // 0x01BD3D3C
std::vector<CIOObject*> g_vObject;  // 0x01BD3D4C

// 0x00403AE0
void MessageLoop_common(HACCEL hAccTable)
{
    guard(L"void MessageLoop_common(HACCEL hAccTable)");  // FIXED: added

    tagMSG msg;
    while (::GetMessageW(&msg, 0, 0, 0) != 0)
    {
        if (!::TranslateAcceleratorW(msg.hwnd, hAccTable, &msg))
        {
            ::TranslateMessage(&msg);
            ::DispatchMessageW(&msg);
        }
    }

    unguard();
}

// 0x00403B50
void MessageLoop_mailing(HACCEL hAccTable)
{
    _BEFORE

    MessageLoop_common(hAccTable);

    _AFTER_FIN
}

// 0x00403C20
uint32_t __stdcall ServerListeningThread(void*)
{
    CThreadManager::RegisterThisThread(ThreadIndex_ListenThread);

    if (Config::ExceptionMailing())
    {
        ServerListeningThread_mailing();
    }
    else
    {
        ServerListeningThread_common();
    }

    return 0;
}

// inlined
void ServerListeningThread_common()
{
    guard(L"void ServerListeningThread_common()");  // FIXED: added

    WSAData wsaData;
    int wsaStartupResult = ::WSAStartup(WINSOCK_VERSION, &wsaData);
    if (wsaStartupResult != 0)
    {
        g_winlog.Add(LOG_ERROR, "WSAStartup error 0x%x", wsaStartupResult);
        unguard();
        return;
    }

    uint32_t ioThreadNumber = CThreadManager::GetMaxIOThreadNumber();
    Threads::CreateIOThread(ioThreadNumber);

    // g_logServer.Run(3999, CServerSocket::Allocate, false, 0, 0);  FIXED
    g_logServer.Run(Config::s_port, CServerSocket::Allocate, false, 0, 0);

    Threads::WaitThread();

    unguard();
}

// inlined
void ServerListeningThread_mailing()
{
    _BEFORE

    ServerListeningThread_common();

    _AFTER_FIN
}

// 0x0040AF40
void CreateIOThread(uint32_t maxIOThread)
{
    guard(L"BOOL CreateIOThread(int nThread)");

    Threads::g_bTerminating = false;
    Threads::s_timer = ::CreateEventW(0u, 0, 0, 0u);

    IOCP::g_hCompletionPort = (HANDLE)::CreateIoCompletionPort((HANDLE)INVALID_HANDLE_VALUE, 0u, 0u, 0u);
    IOCP::g_hCompletionPortExtra = (HANDLE)::CreateIoCompletionPort((HANDLE)INVALID_HANDLE_VALUE, 0u, 0u, 0u);

    SYSTEM_INFO systemInfo;
    ::GetSystemInfo(&systemInfo);
    int numberOfProcessors = systemInfo.dwNumberOfProcessors;

    for (uint32_t i = 0; i < maxIOThread; ++i)
    {
        unsigned int threadId;
        HANDLE thread = (HANDLE)_beginthreadex(NULL, 0, &Threads::IOThread, (void*)i, 0, &threadId);

        ThreadIndices index = static_cast<ThreadIndices>(i);
        CThreadManager::SetThread(index, thread);

        int affinityMask = 1 << (i % numberOfProcessors);
        ::SetThreadAffinityMask(thread, affinityMask);
    }

    unguard();
}

// 0x0040ABB0
void WaitThread()
{
    g_deadlock_checker.Init();

    bool sleep = g_vHandle.size() == 0;

    while (true)
    {
        guard(L"unsigned __stdcall WaitThread(void *)");

        long dwTick = ::GetTickCount();
        if (dwTick > (g_deadlock_checker.lastTick() + 180 * 1000))
        {
            guard(L"!!! deadlock or super-lag detected !!!");
            g_winlog.Add(LOG_ERROR, L"!!! deadlock or super-lag detected!!!  (diff %d sec) ", (dwTick - g_deadlock_checker.lastTick()) / 1000);

#ifdef NDEBUG
            CRASH;
#endif
            unguard();
        }

        LONG nMinNextTime = ::GetTickCount() + 2 * CIO_DEADLOCK_CHECKER_INTERVAL;
        LONG nWait = 2 * CIO_DEADLOCK_CHECKER_INTERVAL;

        CIOObject::TimerDispatch();

        if ((nMinNextTime - ::GetTickCount()) >= 100)
        {
            nWait = 100;
        }
        else
        {
            nWait = nMinNextTime - ::GetTickCount();
        }

        if (sleep)
        {
            ::Sleep(nWait);
        }
        else
        {
            DWORD dwWaitResult = ::WaitForMultipleObjects(g_vHandle.size(), &g_vHandle[0], FALSE, nWait);
            if ((dwWaitResult >= WAIT_OBJECT_0) && (dwWaitResult < WAIT_OBJECT_0 + g_vHandle.size()))
            {
                dwWaitResult -= WAIT_OBJECT_0;
                CIOObject* object = g_vObject[dwWaitResult];
                object->OnWaitCallback();
            }
        }

        if (Threads::g_bTerminating)
        {
            CRASH;
        }

        unguard();
    }
}

// 0x0040AE60
void IOThread_mailing(ThreadIndices index)
{
    _BEFORE

    IOThread_common(index);

    _AFTER_FIN
}

// 0x0040AD20
void Threads::IOThread_common(ThreadIndices /*index*/)
{
    guard(L"void IOThread_common(void *arglist)");

    int ioBufferCount = ::GetPrivateProfileIntW(L"Setting", L"IOBufferCount", 20000, L".\\l2server.ini");

    CIOBufferPool* pool = new CIOBufferPool(ioBufferCount / CThreadManager::GetMaxIOThreadNumber());
    g_IOBufferPools[tls::GetCurrentThreadIndex()] = pool;

    DWORD nextSyncTime = ::GetTickCount();
    HANDLE completionPort = IOCP::g_hCompletionPort;

    std::srand((int)std::time(NULL));

    static const int IOC_WAIT = 50;
    while (true)
    {

        CIOObject* object = NULL;
        DWORD now = ::GetTickCount();
        DWORD waitTick;
        if (nextSyncTime >= now)
        {
            waitTick = nextSyncTime - now;
        }
        else
        {
            CIOObject::TimerDispatch();
            nextSyncTime = ::GetTickCount() + IOC_WAIT;
            waitTick = IOC_WAIT;
        }

        // ::InterlockedIncrement(&CReporter::g_nRunningThread); UNUSED in L2LogD

        DWORD dwTransferred;
        LPOVERLAPPED overlapped;
        BOOL success = ::GetQueuedCompletionStatus(completionPort, &dwTransferred, (PULONG_PTR)&object, &overlapped, waitTick);

        // ::InterlockedDecrement(&CReporter::g_nRunningThread); UNUSED in L2LogD

        if (object != NULL)
        {
            object->OnIOCallback(success, dwTransferred, overlapped);
        }
    }

    unguard();
}

// 0x0040AF00
uint32_t __stdcall Threads::IOThread(void* threadIndex)
{
    int param = reinterpret_cast<int>(threadIndex);
    ThreadIndices index = static_cast<ThreadIndices>(param);
    CThreadManager::RegisterThisThread(index);

    if (Config::ExceptionMailing())
    {
        IOThread_mailing(index);
    }
    else
    {
        IOThread_common(index);
    }

    return 0;
}

// 0x0040CEC0
uint32_t __stdcall CliLogThread(void*)
{
    CThreadManager::RegisterThisThread(ThreadIndex_CliLog);

    CIOBufferPool* pool = new CIOBufferPool(50);

    g_IOBufferPools[ThreadIndex_CliLog] = pool;

    if (Config::ExceptionMailing())
    {
        Threads::CliLogThread_mailing();
    }
    else
    {
        Threads::CliLogThread_common();
    }

    return 0;
}

// 0x0040C4B0
void CliLogThread_common()
{
    guard(L"void CliLogThread_common()");  // FIXED: added

    CLogCli::Job();  //  INLINED COMMON

    unguard();
}

// 0x0040C420
void CliLogThread_mailing()
{
    _BEFORE

    Threads::CliLogThread_common();

    _AFTER_FIN
}

// 0x00404340
uint32_t __stdcall SqlQueueThread(void*)
{
    // CThreadManager::RegisterThisThread(ThreadIndex_Log); FIXED
    CThreadManager::RegisterThisThread(ThreadIndex_SqlQueue);
    if (Config::ExceptionMailing())
    {
        SqlQueueThread_mailing();
    }
    else
    {
        SqlQueueThread_common();
    }

    return 0;
}

// 0x004042A0
void SqlQueueThread_mailing()
{
    _BEFORE

    SqlQueueThread_common();

    _AFTER_FIN
}

// 0x00404330
void SqlQueueThread_common()
{
    guard(L"void SqlQueueThread_common()");  // FIXED: added

    CSQLQueue::Job();

    unguard();
}

// 0x00404270
uint32_t __stdcall PerfMonitorThread(void*)
{
    CThreadManager::RegisterThisThread(ThreadIndex_PerfMonitor);
    if (Config::ExceptionMailing())
    {
        PerfMonitorThread_mailing();
    }
    else
    {
        PerfMonitorThread_common();
    }

    return 0;
}

// 0x004041D0
void PerfMonitorThread_mailing()
{
    _BEFORE

    PerfMonitorThread_common();

    _AFTER_FIN
}

// 0x00404260
void PerfMonitorThread_common()
{
    guard(L"void PerfMonitorThread_common()");  // FIXED: added

    PerfMonitor::Job();

    unguard();
}

// 0x00404420
uint32_t __stdcall BCPThread(void* bcpSlot)
{
    int slot = reinterpret_cast<int>(bcpSlot);
    CThreadManager::RegisterThisThread((ThreadIndices)(ThreadIndex_IO8 - slot));
    if (Config::ExceptionMailing())
    {
        Threads::BCPThread_mailing(slot);
    }
    else
    {
        Threads::BCPThread_common(slot);
    }

    return 0;
}

// 0x00404370
void BCPThread_mailing(int bcpSlot)
{
    _BEFORE

    BCPThread_common(bcpSlot);

    _AFTER_FIN
}

// 0x00404370
void BCPThread_common(int bcpSlot)
{
    guard(L"void BCPThread_common(int bcpSlot)");  // FIXED: added

    CBCPLogFile::Job(bcpSlot);

    unguard();
}

}  // namespace Threads
