#include "utils/Utils.h"
#include "utils/dbg/StackGuard.h"

#include <windows.h>

#include <cstring>
#include <cstdlib>
#include <string>

namespace Utils
{

// 0x00402920
wchar_t* GetFileName(wchar_t* str)
{
    wchar_t* backSlash = ::wcsrchr(str, '\\');
    if (backSlash)
    {
        return backSlash + 1;
    }

    return str;
}

// 0x004062E0
int AnsiToUnicode(const char* ansistr, int /*maxansilen*/, wchar_t* unistr, int /*maxunilen*/)
{
    guard(L"int AnsiToUnicode(char* ansistr, int maxansilen, wchar_t* unistr, int maxunilen)");

    int length = ::MultiByteToWideChar(CP_ACP, 0, ansistr, -1, 0, 0);
    ::MultiByteToWideChar(CP_ACP, 0, ansistr, -1, unistr, length);
    unistr[length - 1] = 0;

    unguard();

    return length;
}

// 0x004063A0
bool IsNumber(wchar_t chr)
{
    return chr >= '0' && chr <= '9' || chr == '-';
}

// 0x00406B40
int GetIntFromString(const wchar_t* strWithNum)
{
    guard(L"int GetIntFromString(wchar_t * sNum)");

    std::wstring str;
    int strWithNumLen = ::wcslen(strWithNum);
    if (strWithNumLen < 1)
    {
        unguard();
        return 0;
    }

    int i = 0;
    if (IsNumber(*strWithNum))
    {
        do
        {
            i++;
            if (i > strWithNumLen)
            {
                break;
            }
            if (i >= 10)
            {
                break;
            }
            str.assign(1, *strWithNum);
        } while (IsNumber(strWithNum[i]));
    }

    int number = ::_wtol(str.c_str());

    unguard();

    return number;
}

// 0x0040BE10
char* UnicodeToMultibyte(const wchar_t* wideMsg)
{
    guard(L"UnicodeToMultibyte");

    int newLength = ::WideCharToMultiByte(CP_ACP, 0, wideMsg, -1, 0, 0, 0, 0);
    char* multiByte = new char[newLength + 1];
    ::WideCharToMultiByte(CP_ACP, 0, wideMsg, -1, multiByte, newLength, 0, 0);
    multiByte[newLength - 1] = 0;

    unguard();

    return multiByte;
}

// 0x0040FC90
char* UnicodeToMultibyte2(const wchar_t* wideMsg)
{
    guard(L"UnicodeToMultibyte");

    int newLength = ::WideCharToMultiByte(CP_ACP, 0, wideMsg, -1, 0, 0, 0, 0);
    char* multiByte = new char[newLength + 1];

    BOOL usedDefaultChar;
    ::WideCharToMultiByte(CP_ACP, 0, wideMsg, -1, multiByte, newLength, 0, &usedDefaultChar);
    multiByte[newLength - 1] = 0;

    unguard();

    return multiByte;
}

// 0x00402690
char* UnicodeToMultibyte_WithoutAllocation(const wchar_t* wideMsg)
{
    static char localBuffer[40001];
    int newLength = ::WideCharToMultiByte(CP_THREAD_ACP, 0, wideMsg, -1, 0, 0, 0, 0);

    BOOL usedDefaultChar;
    ::WideCharToMultiByte(CP_THREAD_ACP, 0, wideMsg, -1, localBuffer, newLength, 0, &usedDefaultChar);
    localBuffer[newLength - 1] = 0;
    return localBuffer;
}

// 0x00413C15
wchar_t* ToWString(int number, wchar_t* outStr, int maxSize)
{
    char buffer[20];
    ::_itoa(number, buffer, maxSize);
    ::mbstowcs(outStr, buffer, sizeof(buffer));
    return outStr;
}

// 0x00406280
int UnicodeToAnsi(const wchar_t* unistr, int /*maxunilen*/, char* ansistr, int /*maxansilen*/)
{
    guard(L"int UnicodeToAnsi(WCHAR* unistr, int maxunilen, char* ansistr, int maxansilen)");

    int length = ::WideCharToMultiByte(CP_ACP, 0, unistr, -1, 0, 0, 0, 0);

    ::WideCharToMultiByte(CP_ACP, 0, unistr, -1, ansistr, length, 0, 0);
    ansistr[length - 1] = 0;

    unguard();

    return length;
}

// 0x00413D9D
wchar_t* TimeToWStr(const std::time_t* time)
{
    tm* localTime = std::localtime(time);
    if (localTime != NULL)
    {
        return ::_wasctime(localTime);
    }

    return NULL;
}

// 0x004126D0
char* TimeToStr(const std::time_t* time)
{
    tm* localTime = std::localtime(time);
    if (localTime != NULL)
    {
        return ::asctime(localTime);
    }

    return NULL;
}

// 0x00406340
void Utils::CopyStrInLowerCase(const wchar_t* in, wchar_t* out)
{
    int inputSize = ::wcslen(in);
    if (inputSize <= 0)
    {
        // out[inputSize] = 0; FIXED
        out[0] = 0;
        return;
    }

    for (int i = 0; i < inputSize; ++i)
    {
        wchar_t symbol = in[i];
        if (symbol > 64u && symbol < 91u)  // CAPITAL letter
        {
            symbol += 32;
        }
        *out = symbol;
        ++out;
    }
}

}  // namespace Utils
